* Motorola Flash Innovation 2003 Contest Project F193
*
* data2cc.asm	for 68HC908QY4		Copyright 2003 Jed Margolin
************************************************************************
* Permission is granted to individuals to make copies for their own use.
* Any commercial use requires permission of the copyright holder.
************************************************************************
*
* Real-time Clock, Reads and displays adc0 and adc1 numerically in Closed
* Captioned Video and with video bars.
*
* Hardware: "Closed Captioned Video (With Data Inputs)"
*
* adc0 and adc1 are read once in video every frame (29.762 Hz)
*
* CC Video is updated once/second and has a range of 0-255.
*
* The video bars are updated once/frame and is scaled to fit the display.
* 
* QY4 Clock is 32MHz with 512 cycles/Horizontal line.
* HSync is 15.625 KHz.
*
* The Realtime clock is corrected for the 15625/525 = 29.762 Hz Frame Rate
*
#pagelength 65t
#pagewidth 124t
$base 10t

NVL  EQU	253	; 10 for testing VBI, 253 for real sync
TIMECAL		EQU	126

$INCLUDE 'qy4equ.h'    	; Equates for MC68HC908QY4 MCU
*====================================================
	ORG	RAMstart

vcount		RMB    1
vline		RMB    1   ; vertical line number in Field
vlfunction	RMB    2   ; address of video line function to be performed
timer_vec	RMB    2   ; Timer Interrupt Redirection Address
cc_data		RMB    2   ; closed-captioning data
cc_adr_ptr	RMB    2   ; closed-captioning data address pointer
cc_data_ptr	RMB    2   ; closed-captioning data pointer
cc_xcount	RMB    1   ; splits CC data generation among several lines
cc_xptr		RMB    2   ; splits CC data generation among several lines
cc_flag		RMB    1   ; 1 = currently sending a CC message
parity		RMB    1   ; used in generating parity for CC data
time:		RMB    3   ; hours:minutes:seconds
timecount:	RMB    1 
clockflag:	RMB    1   ; used in communicating between clock and clockset
clockcal:	RMB    1   ; calibration factor to correct for frame rate
sw1		RMB    3   ; increment switch: processing block
sw2		RMB    3   ; decrement switch: processing block
ASCT		RMB    6   ; ASCII Time
AMPM		RMB    2   ; am or pm
bcd		RMB    2   ; bcd data
bin		RMB    1   ; binary data
adc0		RMB    3   ; low, high, and difference
adc1		RMB    3   ; low, high, and difference
ASCadc0		RMB    3   ; ADC0 in ASCII
ASCadc1		RMB    3   ; ADC1 in ASCII
yp1		RMB    2   ; video parameter bar1 height
yp2		RMB    2   ; video parameter bar2 height
xp		RMB    1   ; video parameter X Ramp
SEND_CCDATA	RMB    52  ; dynamic execution block to send CC data
*====================================================
* Port B Hardware
CSYNC	 	EQU 	0	; out	Composite Sync
VIDEO	 	EQU 	1	; out	1 = On
nHVIDEO	 	EQU 	2	; out	0 = Half Bright 
FIELD		EQU 	3	; out	1 = Field 1
VBLANK 		EQU 	4	; out	V Blanking
IncTime		EQU 	5	; in	0 = Increment Time
DecTime		EQU 	6	; in	0 = Decrement Time
*====================================================
	ORG	$EE00

* First initialize any control registers and variables
START:  LDHX	#$0100		; locate the stack
	TXS

	RSP

	MOV	#$01,CONFIG1	; disable COP
*------------------------------------------------------------------------
* Clock Source
* External Crystal
www1:	CLR	PORTA	
	BSET	4,DDRA		; PA4 output
	BSET	PA4,PORTA	; to help get crystal started
	MOV	#$3C,PAPUP	; Don't leave any inputs floating

	MOV	#$19,CONFIG2	; reset pin active, external crystal

	BSET	1,OSCSTAT
*------------------------------------------------------------------------
* Initialize I/O Ports
	CLR	PORTB
	CLR	DDRB		; Inputs
	MOV	#$FF,PBPUP	; Enable Port B Pullups 
	MOV	#$1F,DDRB

* Initialize clock variables
	LDHX	#$FF
	STHX	sw1
	STHX	sw2
	CLR	sw1+2
	CLR	sw2+2
	CLR	clockflag
	CLR	cc_flag		; not currently sending a message

	MOV	#$12,bcd		; adc default
	MOV	#$34,bcd+1

* Initialize ADC 
	MOV	#$60,ADICLK	; 7.159 MHz/8 = 894 KHz
	NOP
	MOV	#$00,ADSCR	; AD Channel 0
*	MOV	#$01,ADSCR	; AD Channel 1

	BSET	ADCO,ADSCR	; continuous conversion

	LDHX	#$0000		; zero value
	STHX	adc0	
	STHX	adc1	
	CLR	adc0+2
	CLR	adc1+2
*------------------------------------------------------------------------
	BRA	MAIN
*------------------------------------------------------------------------
*------------------------------------------------------------------------
* NTSC Sync:
*
* In order to do Closed Captioning Clock which requires CC_Clock = 32H:
* Bus Clock is 8.0000 MHz, cycle = 0.125 us  
*
* Horizontal frequency is defined as 512 cycles = 64.00 us = 15.625 KHz
*
* CC_Clock is 14 cycles = 2.000 us = 500.0 KHz
*
* HSync = 4.80 us = 38 clocks
*
* The Interrupts can be set as an integer number of bus clocks.
*
* Everything is done at the interrupt level.
*
* The main line code is just NOPs because they are single cycle instructions.
*
* Some things are done at twice the horizontal rate and require cycle 
* counting.
*
* Interlacing is done as per NTSC spec by moving every other VBI 1/2H early
*
MAIN:	MOV	#NVL,vcount  ; number of active video lines/field
			     ; 10 for testing, 253 for real sync

	MOV	#9,vline     ; number of lines in Vertical Blanking Interval

	JSR	cc_new	     ; set up pointers for CC message

	LDHX	#$0023	     ; default time
	STHX	time
	MOV	#$45,time+2

	MOV	#30,timecount		; 30 fields/sec for the clock
	MOV	#TIMECAL,clockcal	; clock correction for 15.625 KHz

	LDA	#240
	STA	yp1
	STA	yp2
	LDA	#241
	STA	yp1+1
	STA	yp2+1

	MOV	#1,xp

	LDHX	#TIRQ1		; Timer Overflow Vector: Real Horizontal Sync
	STHX	timer_vec

	LDHX	#F2vlf1		; the first video line function in Field 2
	STHX	vlfunction

	JSR	TIRQ_Init	; initialize timer irqs

	JMP	NOPS		; Everything gets done in the timer interrupt 
*=============================================================================
* Initialize Timer Interrupts
TIRQ_Init:

* Timer counter, must write to high byte first, 511 -> 0-511
	LDA	#$01		; 511 = $01FF
	STA	TMODH		

	LDA	#$FF
	STA	TMODL

	LDA	#$00		; PS2, PS1, PS0; TSTOP = 0
	STA	TSC

	BSET	TOIE,TSC	; enable interrupts

	CLI

	RTS
*==========================================================================
* Timebase 						($021C)
* Jump to Address in User's Vector Table
TIM_INT:
*        PSHX		; we don't care about trashing XRegs
*        PSHH

	LDHX	timer_vec
	JMP	,X
*==========================================================================
	ORG	$EF00	; keep the Security Code from changing
UNUSED: RTI
*==========================================================================
***************************************************************************
* Timer Overflow Interrupt (TIM_OF_VEC)
* Active Video: Field 2 begins on Line 271 1/2, ends on Line 525
* Real Horizontal Sync
TIRQ1:	

*---------
* HSYNC will be 4.77 us = 38 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#9		; [2]
tq1a:	DBNZX	tq1a		; [3]
	NOP			; [1]

	BCLR	CSYNC,PORTB	; [4]  SYNC off
* ---------------------------------------------------
* Check Vertical Count
	DBNZ	vcount,tq1c

* We have reached the end of active video. Next, start VBI
	LDHX	#TIRQ2	 ; Timer Overflow Vector: Start Field 1 VBI next time
	STHX	timer_vec

	BSET	VBLANK,PORTB	; oscilloscope trigger high
	BSET	FIELD,PORTB	; Field 1 flag

	BRA	TIRQRET

* Video is still active. We have about 50 us to do something else
tq1c:	JSR	Do_Field2
	BRA	TIRQRET
*======================================================================
NOPORG	EQU	$FB00
NOPSHI	EQU	{NOPORG > 8}		; high byte of .ORG for nops.h
NOPSLO	EQU	NOPORG&$FF		; low byte of .ORG for nops.h
*---------
* RTI to beginning of NOPS to make sure an intterupt does not occur during
* the 2-cycle JMP instruction.
TIRQRET:
	TSX			; 2	SP to H:X

	LDA	#NOPSHI		; 2	NOPS High Address
	STA	3,X		; 4

	LDA	#NOPSLO		; 2	NOPS Low Address
	STA	4,X		; 4

	RTI			; 7	(21)
*==========================================================================
* Field 1 VBI: Leading Equalization Pulses
TIRQ2:	

* Leading Equalization Pulses
* We need to do two equalization pulses per line
*
* we need to align it with HSync
* Line 1
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 512 cycles
* Line 2
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 512 cycles
* Line 3
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 512 cycles
* Field 1 VBI: Vertical Sync Pulses
* We need to do two VSync pulses per line
* Line 4
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 512 cycles
* We need to do two VSync pulses per line
* Line 5
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 512 cycles
* We need to do two VSync pulses per line
* Line 6
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 512 cycles
* Trailing Equalization Pulses
* We need to do two equalization pulses per line
* Line 7
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 512 cycles
* We need to do two equalization pulses per line
* Line 8
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 512 cycles
* We need to do two equalization pulses per line
* Line 9
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ2	
	BCLR	TOF,TSC		; [4] reset interrupt flag

	LDHX	#TIRQ3	    	; Timer Overflow Vector: active video
	STHX	timer_vec
	MOV	#NVL,vcount ; number of vertical lines: 10 test, 253 real sync
	MOV	#9,vline  ; number of lines used in Vertical Blanking Interval

	LDHX	#F1vlf1		; the first video line function for Field 1
	STHX	vlfunction

	JMP	TIRQRET
*======================
* Active Video: Field 1 begins on Line 10, ends on Line 262 1/2
* Real Horizontal Sync
TIRQ3:	

*---------
* HSYNC will be 4.77 us = 34 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on

	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#9		; [2]
tq3a:	DBNZX	tq3a		; [3]
	NOP			; [1]
	BCLR	CSYNC,PORTB	; [4]  SYNC off
* ---------------------------------------------------
* Check Vertical Count
	DBNZ	vcount,tq3c

* We have reached the end of active video. Next, start VBI
	LDHX	#TIRQ4	     ; Timer Overflow Vector: Start VBI next time
	STHX	timer_vec

	BSET	VBLANK,PORTB	; oscilloscope trigger high
	JMP	TIRQRET

* Video is still active
* We have about 50 us to do something else
* But we need to take a shortcut on line 21 (CC Encoder)
tq3c:	LDA	vline
	CMP	#19	; [2]  Closed Captioning line 21
* TV lines start with line 1, we start with line 0. Our current line is 
* line 19 = TV line 20. Therefore the next TV line is 21.
	BNE	tq3d	 

	LDHX	#TIRQ3A	     ; next one is special
	STHX	timer_vec

tq3d:	JSR	Do_Field1
	JMP	TIRQRET
*========================================================================
* Special for CC
* Real Horizontal Sync
TIRQ3A:	

*---------
* HSYNC will be 4.77 us = 38 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on

	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#9		; [2]
tq3a1:	DBNZX	tq3a1		; [3]
	NOP			; [1]
	BCLR	CSYNC,PORTB	; [4]  SYNC off
* ---------------------------------------------------
* Check Vertical Count
	DEC	vcount

* Go back the next time
	LDHX	#TIRQ3	     ; Timer Overflow Vector:
	STHX	timer_vec

* Send CC Data from dynamically assembled table
* Wait to clock run-in
	LDX	#8		; [2]
tq3a3a:	DBNZX	tq3a3a		; [3]	
	NOP			; [1]

* 10.074 us from start of HSYNC (80 cycles)
* Send CC Data

* CC Clock Run-in, 7 cycles of 500 KHz (32H)
	LDX	#6		; [2]
tq3a3:	BSET 	VIDEO,PORTB	; [4]
	BSET 	VIDEO,PORTB	; [4]  waste 4 cycles
	BCLR 	VIDEO,PORTB	; [4]
	NOP			; [1]
	DBNZX	tq3a3		; [3]	

	BSET 	VIDEO,PORTB	; [4]	start 7th run-in clock
	BSET 	VIDEO,PORTB	; [4]  waste 4 cycles

	BCLR 	VIDEO,PORTB	; [4]

* 2-bit '0' 32 cycles from trailing edge of last run-in clock to start bit
* (zero-crossing of where a sine wave would be)
	LDX	#5		; [2]
tq3a4:	DBNZX	tq3a4		; [3]	
	NOP			; [1]
	NOP			; [1]
* Now send out the data - 16 cycles/bit
* The only way to do this fast enough is to use a table of instructions that
* was made from the data to be sent. The table was made before this vertical 
* line.
	JSR	SEND_CCDATA

* We will not be returning in time to catch the next Timer Interrupt
* We will send the next line ourself
* Need next HSYNC to be at 512 cycles

* Make it come out to 512 cycles
	LDX	#4	; [2]
tq3a5:	DBNZX	tq3a5	; [3]
	NOP		; [1]
*===
* Start next video line at 512 cycles
* HSYNC will be 4.77 us = 38 cycles
	BSET	CSYNC,PORTB	; [4]  HSYNC on

	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#9		; [2]
tq3a6:	DBNZX	tq3a6		; [3]
	NOP			; [1]
	BCLR	CSYNC,PORTB	; [4]  HSYNC off
* ---------------------------------------------------
	DEC	vcount

	JMP	TIRQRET
*========================================================================
* Active Video: end of Field 1
* Start Field 2 VBI 
* Do 1/2 Horizontal line and 1 Equalizing pulse
* Real Horizontal Sync
TIRQ4:	

* Line 262
*---------
* HSYNC will be 4.77 us = 38 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on

	BCLR	TOF,TSC		; [4]  reset interrupt flag

	LDX	#9		; [2]
tq4a:	DBNZX	tq4a		; [3]
	NOP			; [1]
	BCLR	CSYNC,PORTB	; [4]  SYNC off
* ---------------------------------------------------
* Bring cycle count from CSYNC to 256 (512/2)
	LDX	#69
tq4b:	DBNZX	tq4b
	NOP
	BCLR	FIELD,PORTB	; [4] Field 1 flag
*---------
* Do 1 Equalizing Pulse
* Do the pulse and get ready for the next step
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ2

	LDHX	#TIRQ5	    ; Timer Overflow Vector: active video
	STHX	timer_vec

	JMP	TIRQRET
*======================
* Field 2 VBI: Equalization Pulses
TIRQ5:	
* Line 263
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 512 cycles
* Line 264
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

* end at 512 cycles
* Line 265
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1

* end at 228 cycles (6 EQ pulses)
* Field 2 VBI: Vertical Sync Pulses
* We need to do two VSync pulses per line
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* Line 266
* end at 512 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 256 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* Line 267
* end at 512 cycles
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 512 cycles
* Line 268
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	VS1

* end at 256 cycles
* Trailing Equalization Pulses
* We need to do two equalization pulses per line
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
* end at 512 cycles
* Line 269
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
* end at 512 cycles
* Line 270
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1	

	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ1
* 
* end at 512 cycles
* Line 271
	BSET	CSYNC,PORTB	; [4]  SYNC on
	JSR	EQ2	
	BCLR	TOF,TSC		; [4] reset interrupt flag

* Start of Field 2 Video
	LDX	#7		; [2]	add 4+4+4
tq5a:	DBNZX	tq5a		; [3]
	NOP			; [1]

	BCLR	CSYNC,PORTB	; [4]  SYNC off

* Wait 
	LDX	#66	  ; [2]
tq5b:	DBNZX	tq5b	  ; [3]
* Line 271 1/2
*---------
* End of trailing equalization pulses
	BCLR	VBLANK,PORTB	; oscilloscope trigger low
	MOV	#9,vline
*------------
* Start Field 2 Active Video, which ends in an integer number of H lines
* and begins Field 1 VBI
	LDHX	#TIRQ1		; Timer Overflow Vector
	STHX	timer_vec

	MOV	#NVL,vcount ; number of vertical lines: 10 test, 253 real sync
	MOV	#9,vline  ; number of lines used in Vertical Blanking Interval

	LDHX	#F2vlf1		; the first video line function for Field 2
	STHX	vlfunction

	JMP	TIRQRET
*==========================================================================
***************************************************************************
* Equalization Pulse
EQ1:	
*	BSET	CSYNC,PORTB	; [4]  SYNC on

	LDX	#3		; [2]
eq1a:	DBNZX	eq1a		; [3]
	BCLR	CSYNC,PORTB	; [4]  SYNC off

* Wait 
	LDX	#75		; [2]
eq1b:	DBNZX	eq1b	   	; [3]
	NOP

	RTS			; [4]
*==========================================================================
* Equalization Pulse
EQ2:	
*	BSET	CSYNC,PORTB	; [4]  SYNC on

	LDX	#3		; [2]
eq2a:	DBNZX	eq2a		; [3]
	BCLR	CSYNC,PORTB	; [4]  SYNC off

	RTS			; [4]
*==========================================================================
* VSync pulse
VS1:	
*	BSET	CSYNC,PORTB	; [4]  SYNC on

	LDX	#69
vs1a:	DBNZX	vs1a	   	; [3]	
	NOP			; [1]
	NOP			; [1]

	BCLR	CSYNC,PORTB	; [4]  SYNC off
	
	LDX	#8		; [3]
vs1b:	DBNZX	vs1b	   	; [3]	
	NOP			; [1]
	NOP			; [1]
	RTS
*==========================================================================
*==========================================================================
* Wait 1000 us
*   cycles = 19 + 8*X ; X = (cycles-19)/8
*
* At 8.0 MHz, we need 1000*8.0 = 8000 cycles
* xval = (time*BClock -19)/8
* time = (xval*8 +19)/BClock
* For BClock = 8.0 MHz, xval = 997
*			JSR		4
WAIT1ms:
        PSHX		;		2
        PSHH		;		2

	LDHX	#997	; 		3

wt1ms	AIX	#-1	;		2
	CPHX	#0000	;		3
	BNE	wt1ms   ;		3

	PULH		;		2
	PULX		;		2
	RTS		;		4
*=============================================================================
TEST1:	BSET	CSYNC,PORTB
	JSR	WAIT1ms
	BCLR	CSYNC,PORTB
	JSR	WAIT1ms
	BRA	TEST1
*=============================================================================
******************************************************************************
* Video Line Functions
******************************************************************************
*=============================================================================
* Field 2 Video Line Functions
Do_Field2:
	BCLR	nHVIDEO,PORTB	; half bright
	LDA	vline

	LDHX	vlfunction
	JMP	,X

* F2vlf1 - line 10: Do the switches and set the time
* F2vlf2 - line 11: Do the clock
* F2vlf3 - line 12: Convert time to ASCII for CC
* F2vlf4 - line 13: Setup adc0 for CC
* F2vlf5 - line 14: Setup adc1 for CC
* F2vlf6 - line 15: Do the Y Parameters for the video bars
* F2vlf7 - Do the video bars
*===========================
* Read and debounce the switches, set the time
F2vlf1:	CMP	#10
	BNE	f2out

	JSR	Do_Switches	; read the switches and set the time
	LDHX	#F2vlf2		; next function
	STHX	vlfunction

f2out:	INC	vline
	RTS
*===========================
* We are at line 11
* Do the clock
F2vlf2:	JSR	Do_Time		; do the clock

	LDHX	#F2vlf3		; next function
	STHX	vlfunction

	INC	vline
	RTS
*===========================
* We are at line 12
* Convert time to ASCII for CC
F2vlf3:	JSR	TACC		; Time to ASCII

	LDHX	#F2vlf4		; next function
	STHX	vlfunction

	INC	vline
	RTS
*===========================
* We are at line 13
* Setup adc0 for CC
F2vlf4:	JSR	Setup1		; Do ADC0
	MOV	#$01,ADSCR	; AD Channel 1 is next

	LDHX	#F2vlf5		; read it next line
	STHX	vlfunction

	INC	vline
	RTS
*===========================
* We are at line 14
* Setup adc1 for CC
F2vlf5:	JSR	Setup2		; Do ADC1
	MOV	#$0,ADSCR	; AD Channel 0 is next

	LDHX	#F2vlf6		
	STHX	vlfunction

	INC	vline
	RTS
*===========================
* We are at line 15
* Do Video Y Parameters for the video bars
F2vlf6:	

* From ADC0
	LDA	adc0

f2v6a:	LSRA			; scale it to fit two channels
	LSRA
	STA	bin

* Channel 1 Baseline is 150
	LDA	#150
	SUB	bin
	STA	yp1
	MOV	#150,yp1+1

* From ADC1
	LDA	adc1

	LSRA			; scale it to fit two channels
	LSRA
	STA	bin

* Channel 2 Baseline is 225
	LDA	#225
	SUB	bin
	STA	yp2
	MOV	#225,yp2+1

f2v6out:
	LDHX	#F2vlf7		
	STHX	vlfunction

	INC	vline
	RTS
*===========================
* Do the video bars
F2vlf7:	LDX	#2
f2vlf7a	DBNZX	f2vlf7a

	JMP	Bars		; draw the bars
*==========================================================================
* Draw the channel bars
Bars:	BCLR	nHVIDEO,PORTB	; half bright

	CMP	yp1
	BLO	barsout		; not time to draw the first bar yet

	CMP	yp1+1
	BHI	bars2

* Channel 1
* Draw the bar
	LDHX	#4
bt1:	DBNZX	bt1

	BSET 	VIDEO,PORTB

	LDHX	#100
bt2:	DBNZX	bt2

	BCLR 	VIDEO,PORTB

barsout:
	BSET	nHVIDEO,PORTB	; full bright
	INC	vline		; for next time
	RTS

* Channel 2
bars2:	CMP	yp2
	BLO	barsout

	CMP	yp2+1
	BHI	barsout		; we are done drawing the last bar

* Channel 2
* Draw the bar
	BSET 	VIDEO,PORTB

	LDHX	#100
bt3:	DBNZX	bt3

	BCLR 	VIDEO,PORTB

	BRA	barsout
*=============================================================================
*=============================================================================
* Field 1 Video Line Functions
Do_Field1:
	BCLR	nHVIDEO,PORTB	; half bright
	LDA	vline

	LDHX	vlfunction
	JMP	,X

* F1vlf1 - line 10: Get the next CC data and make odd parity
* F1vlf2 - line 11: Assemble the cc data into an executable table (part 1)
* F1vlf3 - line 12: Assemble the cc data into an executable table (part 2)
* F1vlf4 - line 13: Assemble the cc data into an executable table (part 3)
* F1vlf5 - line 14: Assemble the cc data into an executable table (part 4)
* F1vlf6 - Do the bars
*===========================
* CC Messages
F1vlf1:	CMP	#10
	BNE	F1vlf1out

cc_continue:
* If we are currently sending a message, then continue to send it
	BRSET	0,cc_flag,ccon2

* If we are not currently sending a message, should we start one?
	LDA	timecount	; did we just update the clock?
	CMP	#30
	BEQ	ccon1		; yes

* If we're not doing or starting a message, skip the remaining CC functions
* (Or we might have gotten here because the message just ended)
end_msg:
	CLR	cc_flag	  

	LDHX	#F1vlf7		; next vertical line function (the bars)
	STHX	vlfunction
	INC	vline		; for next time
	RTS

* Start the CC message
ccon1:	JSR	cc_new		; start the message and set the cc_flag

* Get the next CC data and make odd parity
* first word
ccon2:	LDHX	cc_adr_ptr	; points to next data address

	LDA	,X
	STA	cc_data_ptr

	LDA	1,X
	STA	cc_data_ptr+1

	LDHX	cc_data_ptr	; points to data
	BEQ	end_msg		; end of a message

	LDA	,X		; next data
	STA	cc_data

	LDHX	cc_adr_ptr	; update the pointer to the next data address
	AIX	#2		
	STHX	cc_adr_ptr

* second word
	LDA	,X
	STA	cc_data_ptr

	LDA	1,X
	STA	cc_data_ptr+1

	LDHX	cc_data_ptr	; points to data
	BEQ	end_msg		; end of a message

	LDA	,X		; next data
	STA	cc_data+1

	LDHX	cc_adr_ptr	; update the pointer
	AIX	#2		; next data
	STHX	cc_adr_ptr

* The data are in cc_data, cc_data+1
* Do odd parity for cc_data
par0:	LDX	#7
	LDA	cc_data
	CLR	parity

par1:	LSRA
	BCC	par2
	INC	parity

par2:	DBNZX	par1

* If the sum of the bits is already odd, d0 will be 1 (we are done)
	BRSET	0,parity,par3

* If not set d7 to '1'
	BSET	7,cc_data

* Do odd parity for cc_data+1
par3:	LDX	#7
	LDA	cc_data+1
	CLR	parity
par4:	LSRA
	BCC	par5
	INC	parity
par5:	DBNZX	par4

* If the sum of the bits is already odd, d0 will be 1 (we are done)
	BRSET	0,parity,par6

* If not set d7 to '1'
	BSET	7,cc_data+1

* We are done with parity and done for this line
par6:	LDHX	#F1vlf2		; next vertical line function 
	STHX	vlfunction

F1vlf1out:
	INC	vline		; for next time
	RTS
*===========================
* We are at line 11
* Assemble the cc data into an executable table (part 1)
F1vlf2:

* Assemble the cc data into an executable table 
	LDHX	#SEND_CCDATA
*---
* Make a 'BSET	VIDEO,PORTB' instruction Start Bit
	LDA	CC_one_Instr
	STA	,X

	LDA	CC_one_Instr+1
	STA	1,X

	LDA	CC_one_Instr+2
	STA	2,X

	AIX	#3		; increment table pointer

	STHX	cc_xptr
*---
* We don't have enough time to do all on one line so we will split it up
	MOV	#4,cc_xcount	; 5 bits
	JSR	DO_XTABLE

* We are done for this line
	LDHX	#F1vlf3		; next vertical line function 
	STHX	vlfunction

	INC	vline		; for next time
	RTS
*===========================
* We are at line 12
* Assemble the cc data into an executable table (part 2)
F1vlf3:

* Assemble the cc data into an executable table 
* (cc_xptr points to the next location)
	MOV	#4,cc_xcount	; 5 bits
	JSR	DO_XTABLE

* We are done for this line
	LDHX	#F1vlf4		; next vertical line function 
	STHX	vlfunction
	INC	vline		; for next time
	RTS
*===========================
* We are at line 13
* Assemble the cc data into an executable table (part 3)
F1vlf4:	

* Assemble the cc data into an executable table 
* (cc_xptr points to the next location)
	MOV	#4,cc_xcount	; 5 bits
	JSR	DO_XTABLE

* We are done for this line
	LDHX	#F1vlf5		; next vertical line function 
	STHX	vlfunction
	INC	vline		; for next time
	RTS
*===========================
* We are at line 14
* Assemble the cc data into an executable table (part 4)
F1vlf5:

* Assemble the cc data into an executable table 
* (cc_xptr points to the next location)
	MOV	#4,cc_xcount	; 1 bit
	JSR	DO_XTABLE

* Make a 'BCLR  VIDEO,PORTB' instruction
	LDA	CC_BCLR_Instr
	STA	,X

	LDA	CC_BCLR_Instr+1
	STA	1,X

* Make a 'INC  vline' instruction
	LDA	CC_INC_Instr
	STA	2,X

	LDA	CC_INC_Instr+1
	STA	3,X

* Make an 'RTS' instruction
	LDA	CC_RTS_Instr
	STA	4,X

* We are done for this line
	LDHX	#F1vlf7		; next vertical line function 
	STHX	vlfunction
	INC	vline		; for next time
	RTS
*===========================
* Do the Bars
F1vlf7:	JMP	Bars
*==========================================================================
*==========================================================================
* Closed Captioning Message Tables
CCTest:		FCB   $14,$29,$14,$29,$11,$22,$11,$40,$11,$40  
		FCB   'Hello Dere',$14,$24,$00	    ; and delete to end of row

CCT3:		FCB   $11,$60,'This is a test',$14,$24,$00

* It would be easier to assemble the mesage in RAM, but RAM is limited.

* Paint-On Mode seems to work better for this application.
* Preamble
PRE:	FCB   $14,$29	; Resume Direct Captioning
	FCB   $11,$20   ; white (default)
	FCB   $11,$40   ; white, Row 1

POST:	FCB   $14,$24	; Delete to End of Row

COLON:	FCB   $3A
SPACE:	FCB   $20

* Requires even number of bytes
CCMSG:  FDB   PRE+0,PRE+1,PRE+0,PRE+1			; repeat commands
	FDB   PRE+4,PRE+5,PRE+4,PRE+5
	FDB   ASCT+0,ASCT+1,COLON			; ASCII Time
	FDB   ASCT+2,ASCT+3,COLON
	FDB   ASCT+4,ASCT+5,SPACE,AMPM,AMPM+1,SPACE	  
	FDB   SPACE,ASCadc0+0,ASCadc0+1,ASCadc0+2	; ASCII adc0
	FDB   SPACE,ASCadc1+0,ASCadc1+1,ASCadc1+2	; ASCII adc1
	FDB   POST+0,POST+1,POST+0,POST+1
	FDB   $0000					  ; end of message
*-----------------------------------------------
* Start a new CC message 
cc_new:
	LDA	#(CCMSG&$FF)	; low byte of address
	STA	cc_adr_ptr+1

	LDA	#{CCMSG > 8}	; high byte of address
	STA	cc_adr_ptr

	MOV	#$FF,cc_flag	; send the CC message
	RTS
*-----------------------------------------------
CC_one_Instr:
	JSR	CC_one		; [5 ext]
CC_one:	
	BSET	VIDEO,PORTB	; [4]
	NOP			; [1]
	NOP			; [1]
	NOP			; [1]
	RTS			; [4]
*---
CC_zero_Instr:
	JSR	CC_zero		; [5 ext]
CC_zero:
	BCLR	VIDEO,PORTB	; [4]
	NOP			; [1]
	NOP			; [1]
	NOP			; [1]
CC_RTS_Instr:
	RTS			; [4]
*---
CC_BCLR_Instr:
	BCLR	VIDEO,PORTB
*---
CC_INC_Instr:
	INC	vline
*---
CC_RTI_Instr:
	RTI	
*=============================================================================
* Assemble the cc data into an executable table 
DO_XTABLE:
	LDHX	cc_xptr

xcc3:	LSR	cc_data+1	; second byte
	ROR	cc_data		; first byte
	BCC	xcc4		; 
*---
* Make a 'BSET	VIDEO,PORTB' instruction
	LDA	CC_one_Instr
	STA	,X

	LDA	CC_one_Instr+1
	STA	1,X

	LDA	CC_one_Instr+2
	STA	2,X

	AIX	#3		; increment table pointer

	BRA	xcc5		; [3]
*---
xcc4:	
* Make a 'BCLR	VIDEO,PORTB' instruction
	LDA	CC_zero_Instr
	STA	,X

	LDA	CC_zero_Instr+1
	STA	1,X

	LDA	CC_zero_Instr+2
	STA	2,X

	AIX	#3		; increment table pointer
*---
xcc5:	DEC	cc_xcount
	BNE	xcc3

	STHX	cc_xptr	   ; so we can do more on the next H line

	RTS
*==========================================================================
***************************************************************************
* Clock Functions
***************************************************************************
*==========================================================================
* Read and debounce the switches, set the clock as required
Do_Switches:

dsw1:	LDA	sw1
	BRSET	IncTime,PORTB,dsw1a  ; sets the carry to the state of switch
dsw1a:	ROLA			; shifts Carry into sw1
	STA	sw1
	AND	#$07
	BEQ	dsw1b		; do we have all '0's in low bits?

	CMP	#$07		; do we have all '1's in low bits?
	BNE	dsw2		; next switch
	STA	sw1+1		; debounced switch = off
	CLR	sw1+2		; reset the switch duration counter
	BRA	dsw2		; switch is not pressed

* switch is pressed
dsw1b:	LDX	sw1+2		; increment the switch duration counter
	INCX
	CMPX	#30		; duration is 30 frames
	BMI	dsw1f		; short time

* switch has been pressed for a long time
* increment minutes
	CLR	sw1+2		; reset the duration counter
	JSR	Inc_Hrs
	BRA	dsw3		; only 1 switch at a time

dsw1f:	STX	sw1+2		; save the switch duration counter
	COMA
	EOR	sw1+1		; previous switch state
	AND	#$07
	BNE	dsw2		; same as last time

	CLR	sw1+1		; debounced switch = on
	
* we had a transition from off-to-on
* switch press positive transition
* switch has been pressed for a short time
* increment minutes
	LDA	#$01
	JSR	Inc_Mins
	BRA	dsw3
*-----
* next switch
dsw2:	LDA	sw2
	BRSET	DecTime,PORTB,dsw2a  ; sets the carry to the state of switch
dsw2a:	ROLA			     ; shifts Carry into sw2
	STA	sw2
	AND	#$07
	BEQ	dsw2b		; do we have all '0's in low bits?

	CMP	#$07		; do we have all '1's in low bits?
	BNE	dsw3
	STA	sw2+1		; debounced switch = off
	CLR	sw2+2		; reset the switch duration counter
	BRA	dsw3		; switch is not pressed

* switch is pressed
dsw2b:	LDX	sw2+2		; increment the switch duration counter
	INCX
	CMPX	#30		; duration is 30 frames
	BMI	dsw2f		; short time

* switch has been pressed for a long time
* decrement minutes
	CLR	sw2+2		; reset the duration counter
	JSR	Dec_Hrs
	BRA	dsw3

dsw2f:	STX	sw2+2
	COMA
	EOR	sw2+1		; previous switch state
	AND	#$07
	BNE	dsw3		; same as last time

	CLR	sw2+1		; debounced switch = on

* switch has been pressed for a short time
* increment minutes
* decrement minutes
	LDA	#$99		; add 100 - 1
	JSR	Dec_Mins

dsw3:	RTS
*==========================================================================
Do_Time:
* Do Time
	
* If the time has just been set, reset the frame count
	BRCLR	0,clockflag,dt1	

	BCLR	0,clockflag
	MOV	#30,timecount		; 30 frames/sec
	MOV	#TIMECAL,clockcal	; clock correction for 15.625 KHz
	BRA	dtout	

* frame count
dt1:	DEC	timecount
	BNE	dt2
	BSR	Inc_Secs
	MOV	#30,timecount		; 30 frames/sec
	
* Check if this is a correction cycle
dt2:	DEC	clockcal
	BNE	dtout			; nope
	MOV	#TIMECAL,clockcal	; clock correction for 15.625 KHz

* Apply the correction
	DEC	timecount
	BNE	dtout

	BSR	Inc_Secs
	MOV	#30,timecount		; 30 frames/sec

dtout:	RTS
*==========================================================================
Inc_Secs:
* Increment the seconds
	LDA	time+2
	ADD	#1
	DAA
	STA	time+2
	
	CMP	#$60
	BNE	isout

	CLR	time+2
*----
* minutes
	LDA	time+1
	ADD	#1
	DAA
	STA	time+1
	
	CMP	#$60
	BNE	isout

	CLR	time+1

* hours
	LDA	time
	ADD	#1
	DAA
	STA	time

	CMP	#$24
	BNE	isout
	CLR	time

isout:	RTS
*------------------------
* Time to ASCII for Closed Captioning
TACC:	LDA	time		; Hours,  convert to 12 hour time 
	CMP	#$12
	BLE	dtcc1
	ADD	#$88		; add 100-12
	DAA

dtcc1:	TSTA
	BNE	dtcc2
	LDA	#$12		; 00 hours is 12 am 

dtcc2:	TAX			; save it
	LSRA
	LSRA
	LSRA
	LSRA
	ADD	#$30
	STA	ASCT+0

	TXA			; time
	AND	#$0F
	ADD	#$30
	STA	ASCT+1

	LDA	time+1		; Minutes
	LSRA
	LSRA
	LSRA
	LSRA
	ADD	#$30
	STA	ASCT+2

	LDA	time+1
	AND	#$0F
	ADD	#$30
	STA	ASCT+3

	LDA	time+2		; Seconds
	LSRA
	LSRA
	LSRA
	LSRA
	ADD	#$30
	STA	ASCT+4

	LDA	time+2
	AND	#$0F
	ADD	#$30
	STA	ASCT+5

* am or pm
	LDA	time
	CMP	#$12
	BLT	dtcc3

	LDHX	#$616D		; AM
	BRA	dtcc4

dtcc3:	LDHX	#$706D		; PM
dtcc4:	STHX	AMPM

taccout:	RTS
*==========================================================================
Inc_Mins:
* increment minutes
	ADD	time+1
	DAA
	STA	time+1
	
	CMP	#$60
	BLT	ims1

	SUB	#$60
	STA	time+1

* hours
	LDA	time
	ADD	#1
	DAA
	STA	time

	CMP	#$24
	BNE	ims1
	CLR	time

* zero the seconds
ims1:	CLR	time+2
	BSET	0,clockflag	; clock has just been set
	RTS
*----------------------------------
* Decrement minutes
* (DAA doesn't work with SUB)
Dec_Mins:
	ADD	time+1
	DAA
	STA	time+1

	CMP	#$60
	BCS	dsms2

	SUB	#$40
	STA	time+1

* hours
dsms1:	LDA	time
	ADD	#$99		; subtract 1 hour
	DAA
	STA	time

	CMP	#$00
	BPL	dsms2
	MOV	#$23,time

* zero the seconds
dsms2:	CLR	time+2
	BSET	0,clockflag	; clock has just been set
	RTS
*----------------------------------
Inc_Hrs:
* increment hours
	LDA	time
	ADD	#1
	DAA
	STA	time

	CMP	#$24
	BNE	ihs1
	CLR	time

* zero the seconds
ihs1:	
*	CLR	time+2
	BSET	0,clockflag	; clock has just been set
	RTS
*----------------------------------
* Decrement hours
* (DAA doesn't work with SUB)
Dec_Hrs:
	LDA	time
	ADD	#$99		; subtract 1 hour
	DAA
	STA	time

	CMP	#$00
	BPL	dshs2
	MOV	#$23,time

* zero the seconds
dshs2:	
*	CLR	time+2
	BSET	0,clockflag	; clock has just been set
	RTS
*=============================================================================
* For ADC
* Convert binary to bcd, 255. max
* uses ACC, Xreg, binl, bcd, bcd+1
* input from bin, output in bcd, bcd+1
BIN_BCD:
        CLR     bcd+1
        CLR     bcd
        LDX     #8
bcdloop:
        ROL     bin

	LDA	bcd+1
	ADC	bcd+1
	DAA
	STA	bcd+1

	LDA	bcd
	ADC	bcd
	DAA
	STA	bcd

        DBNZX   bcdloop

        RTS
*==========================================================================
* Read ADC0
Setup1:	

* Read ADC
*	MOV	#123,bin	; for testing
	MOV	ADR,bin		; read ADC
	MOV	bin,adc0
	BSR	BIN_BCD		; Convert to BCD

* Convert to ASCII
	LDA	bcd
	AND	#$0F
	ADD 	#$30
	STA	ASCadc0
	
	LDA	bcd+1
	LSRA
	LSRA
	LSRA
	LSRA
	ADD 	#$30
	STA	ASCadc0+1
	
	LDA	bcd+1
	AND	#$0F
	ADD 	#$30
	STA	ASCadc0+2

	RTS
*-------------------------------------------------------------
* Read ADC1
Setup2:	

* Read ADC
*	MOV	#123,bin	; for testing
	MOV	ADR,bin		; read ADC
	MOV	bin,adc1
	BSR	BIN_BCD		; Convert to BCD

* Convert to ASCII
	LDA	bcd
	AND	#$0F
	ADD 	#$30
	STA	ASCadc1
	
	LDA	bcd+1
	LSRA
	LSRA
	LSRA
	LSRA
	ADD 	#$30
	STA	ASCadc1+1
	
	LDA	bcd+1
	AND	#$0F
	ADD 	#$30
	STA	ASCadc1+2

	RTS
*==========================================================================
******************************************************************************
* NOP Table
******************************************************************************
*==========================================================================
* Put NOPs here
	ORG	NOPORG
$INCLUDE 'nops2.h'   ; 520 NOPs (NOPSHI and NOPSLO must match his address)
*==========================================================================
******************************************************************************
* Vector Table
	ORG	ADCVEC		; A/D conversion complete		
	FDB	UNUSED

	ORG	KBVEC	 	; Keyyboard Vector			
	FDB	UNUSED

	ORG	TIM_OF_VEC  	; Timer Overflow Vector		
	FDB	TIM_INT

	ORG	TIM_CH1_VEC 	; Timer Channel 1 Vector		
	FDB	UNUSED

	ORG	TIM_CH0_VEC	; Timer Channel 2 Vector		
	FDB	UNUSED

	ORG	IRQ_VEC		; IRQ Vector				
	FDB	UNUSED

	ORG	SWI_VEC		; Software Interrupt			
	FDB	UNUSED

	ORG 	RESET_VEC
	FDB	START
*=============================================================================
